use drm::control::Device;

mod card;

fn main() {
  let gpu = match card::Card::open("/dev/dri/card1") {
    Ok(gpu) => gpu,
    Err(err) => {
      panic!("Impossible to open DRM file : {:?}", err);
    }
  };

  println!("{:?}", gpu);

  let resource_handles = gpu.resource_handles().unwrap();

  println!("Connectors:");
  for handle in resource_handles.connectors() {
    match gpu.get_connector(*handle, false) {
      Ok(connector) => {
        println!("\t{} ({:?})", connector.interface().as_str(), connector.state());
        
        let current_encoder = connector
          .current_encoder()
          .and_then(|encoder_handle| gpu.get_encoder(encoder_handle).ok());

        let encoder_info = match current_encoder {
          Some(encoder) => {

            let crtc_info = format_crtc(&gpu, encoder.crtc());

            format!("{:?}, {}", encoder.kind(), crtc_info)
          }
          None => format!("No encoder found")
        };
  
        println!("\t\tEncoder: {}", encoder_info);

        println!(""); // Add blank line
      },
      Err(_) => {}
    }
  }
}

fn format_crtc(gpu: &card::Card, crtc_handle: Option<drm::control::crtc::Handle>) -> String {
  match crtc_handle.and_then(|handle| Some(gpu.get_crtc(handle))) {
    Some(crtc) => format!("{:?}", crtc),
    None => format!("No CRTC data")
  }
}
